==> gcma-0.1.37.tar.gz: gcma-0.1.37/gcma/io/history.py <==
import logging

import numpy as np
import treefiles as tf

from gcma.io.baseio import IOEncoder
from gcma.io.feature import TFeatures, Features
from gcma.io.param import TParams, Params


class Hist:
    def __init__(
        self, params: TParams, features: TFeatures, fvalue: float, it: int = None
    ):
        """
        Stores a generation's best results

        :param params: best parameters of the generation `it`
        :param features: best features of the generation `it`
        :param fvalue: value of the global error
        :param it: iteration
        """
        self.params = params
        self.features = features
        self.fvalue = fvalue
        self.iteration = it

    def to_dict(self) -> dict:
        """
        Dump to dict
        """
        return {
            "iteration": self.iteration,
            "params": self.params,
            "features": self.features,
            "fvalue": self.fvalue,
        }

    @classmethod
    def from_dict(cls, d: dict):
        """
        Load from dict
        """
        return cls(
            Params(d["params"]),
            Features(d["features"]),
            float(d["fvalue"]),
            it=int(d["iteration"]),
        )

    def __repr__(self):
        s = f"Iteration {self.iteration}, fvalue = {self.fvalue}\n"
        s += f"Params:\n{Params(self.params).table()}\n"
        s += f"Features:\n{Features(self.features).table()}"
        return s


class HistoryEncoder(IOEncoder):
    def default(self, obj):
        if isinstance(obj, Hist):
            return obj.to_dict()
        return super().default(obj)


class History(list):
    @property
    def best_iter(self) -> int:
        """
        Return the best iteration
        """
        fvalues = np.array([x.fvalue for x in self])
        return np.argmin(fvalues)

    @property
    def best(self) -> Hist:
        """
        Return the best history among all generations
        """
        return self[self.best_iter]

    @property
    def last(self) -> Hist:
        """
        Return the last generation's history
        """
        return self[-1]

    def add(self, params: TParams, features: TFeatures, fvalue: float):
        """
        Add a history record

        :param params: best parameters of the generation `it`
        :param features: best features of the generation `it`
        :param fvalue: value of the global error
        """
        self.append(Hist(params, features, fvalue, it=len(self)))

    def dump(self, fname: str):
        """
        Export history to json file

        :param fname: filename
        """
        tf.dump_json(fname, self, cls=HistoryEncoder)

    @classmethod
    def load(cls, fname: str):
        """
        Load history from json file

        :param fname: filename
        """
        l = [Hist.from_dict(x) for x in tf.load_json(fname)]
        return History(l)


log = logging.getLogger(__name__)

==> gcma-0.1.41.tar.gz: gcma-0.1.41/gcma/io/history.py <==
import logging

import numpy as np
import treefiles as tf
from treefiles.baseio_.baseio_ import IOEncoder

from gcma.io.feature import TFeatures, Features


class Hist:
    def __init__(
        self, params: tf.TParams, features: TFeatures, fvalue: float, it: int = None
    ):
        """
        Stores a generation's best results

        :param params: best parameters of the generation `it`
        :param features: best features of the generation `it`
        :param fvalue: value of the global error
        :param it: iteration
        """
        self.params = params
        self.features = features
        self.fvalue = fvalue
        self.iteration = it

    def to_dict(self) -> dict:
        """
        Dump to dict
        """
        return {
            "iteration": self.iteration,
            "params": self.params,
            "features": self.features,
            "fvalue": self.fvalue,
        }

    @classmethod
    def from_dict(cls, d: dict):
        """
        Load from dict
        """
        return cls(
            tf.Params(d["params"]),
            Features(d["features"]),
            float(d["fvalue"]),
            it=int(d["iteration"]),
        )

    def __repr__(self):
        s = f"Iteration {self.iteration}, fvalue = {self.fvalue}\n"
        s += f"Params:\n{tf.Params(self.params).table()}\n"
        s += f"Features:\n{Features(self.features).table()}"
        return s


class HistoryEncoder(IOEncoder):
    def default(self, obj):
        if isinstance(obj, Hist):
            return obj.to_dict()
        return super().default(obj)


class History(list):
    @property
    def best_iter(self) -> int:
        """
        Return the best iteration
        """
        fvalues = np.array([x.fvalue for x in self])
        return int(np.argmin(fvalues))

    @property
    def best(self) -> Hist:
        """
        Return the best history among all generations
        """
        return self[self.best_iter]

    @property
    def last(self) -> Hist:
        """
        Return the last generation's history
        """
        return self[-1]

    def add(self, params: tf.TParams, features: TFeatures, fvalue: float):
        """
        Add a history record

        :param params: best parameters of the generation `it`
        :param features: best features of the generation `it`
        :param fvalue: value of the global error
        """
        self.append(Hist(params, features, fvalue, it=len(self)))

    def dump(self, fname: str):
        """
        Export history to json file

        :param fname: filename
        """
        tf.dump_json(fname, self, cls=HistoryEncoder)

    @classmethod
    def load(cls, fname: str):
        """
        Load history from json file

        :param fname: filename
        """
        l = [Hist.from_dict(x) for x in tf.load_json(fname)]
        return History(l)


log = logging.getLogger(__name__)
